@@ -1,9 +1,10 @@
 -- | Stability: provisional
 module Test.Hspec.Core.QuickCheck (
-  modifyMaxSuccess
+  modifyArgs
+, modifyMaxSuccess
 , modifyMaxDiscardRatio
 , modifyMaxSize
-
+, modifyMaxShrinks
 ) where
 
 import           Test.QuickCheck
@@ -30,6 +31,14 @@ modifyMaxSize = modifyArgs . modify
     modify :: (Int -> Int) -> Args -> Args
     modify f args = args {maxSize = f (maxSize args)}
 
+-- | Use a modified `maxShrinks` for given spec.
+modifyMaxShrinks :: (Int -> Int) -> SpecWith a -> SpecWith a
+modifyMaxShrinks = modifyArgs . modify
+  where
+    modify :: (Int -> Int) -> Args -> Args
+    modify f args = args {maxShrinks = f (maxShrinks args)}
+
+-- | Use modified `Args` for given spec.
 modifyArgs :: (Args -> Args) -> SpecWith a -> SpecWith a
 modifyArgs = modifyParams . modify
   where
