@@ -1,19 +1,28 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-module Test.Hspec.Core.RunnerSpec (main, spec) where
+{-# LANGUAGE TypeFamilies #-}
+
+#if MIN_VERSION_base(4,6,0) && !MIN_VERSION_base(4,7,0)
+-- Control.Concurrent.QSem is deprecated in base-4.6.0.*
+{-# OPTIONS_GHC -fno-warn-deprecations #-}
+#endif
+
+module Test.Hspec.Core.RunnerSpec (spec) where
 
 import           Prelude ()
 import           Helper
 
 import           System.IO (stderr)
-import           Control.Monad (replicateM_)
 import           System.Environment (withArgs, withProgName, getArgs)
 import           System.Exit
 import           Control.Concurrent
 import qualified Control.Exception as E
+import           Control.Concurrent.Async
 import           Mock
 import           System.SetEnv
+import           System.Console.ANSI
 
-import           Test.Hspec.FailureReport (FailureReport(..))
+import           Test.Hspec.Core.FailureReport (FailureReport(..))
 import qualified Test.Hspec.Core.Spec as H
 import qualified Test.Hspec.Core.Runner as H
 import qualified Test.Hspec.Core.Formatters as H (silent)
@@ -22,9 +31,6 @@ import qualified Test.Hspec.Core.QuickCheck as H
 import qualified Test.QuickCheck as QC
 import qualified Test.Hspec.Core.Hooks as H
 
-main :: IO ()
-main = hspec spec
-
 quickCheckOptions :: [([Char], Args -> Int)]
 quickCheckOptions = [("--qc-max-success", QC.maxSuccess), ("--qc-max-size", QC.maxSize), ("--qc-max-discard", QC.maxDiscardRatio)]
 
@@ -37,13 +43,36 @@ runPropFoo args = unlines . normalizeSummary . lines <$> do
 spec :: Spec
 spec = do
   describe "hspec" $ do
+
+    let
+      hspec args = withArgs ("--format=silent" : args) . H.hspec
+      hspec_ = hspec []
+
+    it "evaluates examples Unmasked" $ do
+      mvar <- newEmptyMVar
+      hspec_ $ do
+        H.it "foo" $ do
+          E.getMaskingState >>= putMVar mvar
+      takeMVar mvar `shouldReturn` E.Unmasked
+
+    it "runs finalizers" $ do
+      mvar <- newEmptyMVar
+      ref <- newIORef "did not run finalizer"
+      a <- async $ hspec_ $ do
+          H.it "foo" $ do
+            (putMVar mvar () >> threadDelay 10000000) `E.finally`
+              writeIORef ref "ran finalizer"
+      takeMVar mvar
+      cancel a
+      readIORef ref `shouldReturn` "ran finalizer"
+
     it "runs a spec" $ do
-      silence . H.hspec $ do
+      hspec_ $ do
         H.it "foobar" True
       `shouldReturn` ()
 
     it "exits with exitFailure if not all examples pass" $ do
-      silence . H.hspec $ do
+      hspec_ $ do
         H.it "foobar" False
       `shouldThrow` (== ExitFailure 1)
 
@@ -62,7 +91,7 @@ spec = do
           ]
 
     it "stores a failure report in the environment" $ do
-      silence . ignoreExitCode . withArgs ["--seed", "23"] . H.hspec $ do
+      ignoreExitCode . hspec ["--seed", "23"] $ do
         H.describe "foo" $ do
           H.describe "bar" $ do
             H.it "example 1" True
@@ -99,9 +128,9 @@ spec = do
         r <- runPropFoo ["--seed", "42"]
         runPropFoo ["--rerun"] `shouldReturn` r
 
-      forM_ quickCheckOptions $ \(flag, accessor) -> do
-        it ("reuses same " ++ flag) $ do
-          [flag, "23"] `shouldUseArgs` ((== 23) . accessor)
+      forM_ quickCheckOptions $ \(name, accessor) -> do
+        it ("reuses same " ++ name) $ do
+          [name, "23"] `shouldUseArgs` ((== 23) . accessor)
           ["--rerun"] `shouldUseArgs` ((== 23) . accessor)
 
       context "when no examples failed previously" $ do
@@ -140,12 +169,11 @@ spec = do
           r `shouldBe` "WARNING: Could not read environment variable HSPEC_FAILURES; `--rerun' is ignored!\n"
 
 
-    it "does not leak command-line flags to examples" $ do
-      silence . withArgs ["--verbose"] $ do
-        H.hspec $ do
-          H.it "foobar" $ do
-            getArgs `shouldReturn` []
-        `shouldReturn` ()
+    it "does not leak command-line options to examples" $ do
+      hspec ["--diff"] $ do
+        H.it "foobar" $ do
+          getArgs `shouldReturn` []
+      `shouldReturn` ()
 
     context "when interrupted with ctrl-c" $ do
       it "prints summary immediately" $ do
@@ -170,6 +198,16 @@ spec = do
           , ""
           , "  1) foo"
           , ""
+          , "  To rerun use: --match \"/foo/\""
+          , ""
+#if __GLASGOW_HASKELL__ == 800
+          , "WARNING:"
+          , "  Your version of GHC is affected by https://ghc.haskell.org/trac/ghc/ticket/13285."
+          , "  Source locations may not work as expected."
+          , ""
+          , "  Please consider upgrading GHC!"
+          , ""
+#endif
           , "Randomized with seed 23"
           , ""
           ]
@@ -178,7 +216,7 @@ spec = do
         mvar <- newEmptyMVar
         sync <- newEmptyMVar
         threadId <- forkIO $ do
-          silence . H.hspec $ do
+          hspec_ $ do
             H.it "foo" $ do
               putMVar sync ()
               threadDelay 1000000
@@ -187,18 +225,6 @@ spec = do
         throwTo threadId E.UserInterrupt
         takeMVar mvar `shouldReturn` E.UserInterrupt
 
-    context "with --help" $ do
-      let printHelp = withProgName "spec" . withArgs ["--help"] . H.hspec $ pure ()
-      it "prints help" $ do
-        r <- (captureLines . ignoreExitCode) printHelp
-        r `shouldStartWith` ["Usage: spec [OPTION]..."]
-        silence printHelp `shouldThrow` (== ExitSuccess)
-
-      it "constrains lines to 80 characters" $ do
-        r <- (captureLines . ignoreExitCode) printHelp
-        r `shouldSatisfy` all ((<= 80) . length)
-        r `shouldSatisfy` any ((78 <=) . length)
-
     context "with --dry-run" $ do
       let withDryRun = captureLines . withArgs ["--dry-run"] . H.hspec
       it "produces a report" $ do
@@ -243,12 +269,40 @@ spec = do
           , ""
           , "  1) bar"
           , ""
+          , "  To rerun use: --match \"/bar/\""
+          , ""
+#if __GLASGOW_HASKELL__ == 800
+          , "WARNING:"
+          , "  Your version of GHC is affected by https://ghc.haskell.org/trac/ghc/ticket/13285."
+          , "  Source locations may not work as expected."
+          , ""
+          , "  Please consider upgrading GHC!"
+          , ""
+#endif
           , "Randomized with seed 23"
           , ""
           , "Finished in 0.0000 seconds"
           , "2 examples, 1 failure"
           ]
 
+      it "cancels running parallel spec items on failure" $ do
+        child1 <- newQSem 0
+        child2 <- newQSem 0
+        parent <- newQSem 0
+        ref <- newIORef ""
+        ignoreExitCode . hspec ["--fail-fast", "-j", "2"] $ do
+          H.parallel $ do
+            H.it "foo" $ do
+              waitQSem child1
+              "foo" `shouldBe` "bar"
+            H.it "bar" $ do
+              -- NOTE: waitQSem should never return here, as we want to
+              -- guarantee that the thread is killed before hspec returns
+              (signalQSem child1 >> waitQSem child2 >> writeIORef ref "foo") `E.finally` signalQSem parent
+        signalQSem child2
+        waitQSem parent
+        readIORef ref `shouldReturn` ""
+
       it "works for nested specs" $ do
         r <- captureLines . ignoreExitCode . withArgs ["--fail-fast", "--seed", "23"] . H.hspec . removeLocations $ do
           H.describe "foo" $ do
@@ -263,6 +317,16 @@ spec = do
           , ""
           , "  1) foo bar"
           , ""
+          , "  To rerun use: --match \"/foo/bar/\""
+          , ""
+#if __GLASGOW_HASKELL__ == 800
+          , "WARNING:"
+          , "  Your version of GHC is affected by https://ghc.haskell.org/trac/ghc/ticket/13285."
+          , "  Source locations may not work as expected."
+          , ""
+          , "  Please consider upgrading GHC!"
+          , ""
+#endif
           , "Randomized with seed 23"
           , ""
           , "Finished in 0.0000 seconds"
@@ -274,7 +338,7 @@ spec = do
         e1 <- newMock
         e2 <- newMock
         e3 <- newMock
-        silence . withArgs ["-m", "/bar/example"] . H.hspec $ do
+        hspec ["-m", "/bar/example"] $ do
           H.describe "foo" $ do
             H.describe "bar" $ do
               H.it "example 1" $ mockAction e1
@@ -288,7 +352,7 @@ spec = do
         e2 <- newMock
         e3 <- newMock
         e4 <- newMock
-        silence . withArgs ["-m", "/bar/example", "--skip", "example 3"] . H.hspec $ do
+        hspec ["-m", "/bar/example", "--skip", "example 3"] $ do
           H.describe "foo" $ do
             H.describe "bar" $ do
               H.it "example 1" $ mockAction e1
@@ -303,7 +367,7 @@ spec = do
         e1 <- newMock
         e2 <- newMock
         e3 <- newMock
-        silence . withArgs ["-m", "foo", "-m", "baz"] . H.hspec $ do
+        hspec ["-m", "foo", "-m", "baz"] $ do
           H.describe "foo" $ do
             H.it "example 1" $ mockAction e1
           H.describe "bar" $ do
@@ -312,6 +376,26 @@ spec = do
             H.it "example 3" $ mockAction e3
         (,,) <$> mockCounter e1 <*> mockCounter e2 <*> mockCounter e3 `shouldReturn` (1, 0, 1)
 
+    context "with --diff" $ do
+      it "shows colorized diffs" $ do
+        r <- capture_ . ignoreExitCode . withArgs ["--diff", "--color"] . H.hspec $ do
+          H.it "foo" $ do
+            23 `shouldBe` (42 :: Int)
+        r `shouldContain` unlines [
+            red ++ "       expected: " ++ reset ++ red ++ "42" ++ reset
+          , red ++ "        but got: " ++ reset ++ green ++ "23" ++ reset
+          ]
+
+    context "with --no-diff" $ do
+      it "it does not show colorized diffs" $ do
+        r <- capture_ . ignoreExitCode . withArgs ["--no-diff", "--color"] . H.hspec $ do
+          H.it "foo" $ do
+            23 `shouldBe` (42 :: Int)
+        r `shouldContain` unlines [
+            red ++ "       expected: " ++ reset ++ "42"
+          , red ++ "        but got: " ++ reset ++ "23"
+          ]
+
     context "with --format" $ do
       it "uses specified formatter" $ do
         r <- capture_ . ignoreExitCode . withArgs ["--format", "progress"] . H.hspec $ do
@@ -330,7 +414,7 @@ spec = do
     context "with --qc-max-success" $ do
       it "tries QuickCheck properties specified number of times" $ do
         m <- newMock
-        silence . withArgs ["--qc-max-success", "23"] . H.hspec $ do
+        hspec ["--qc-max-success", "23"] $ do
           H.it "foo" $ property $ \(_ :: Int) -> do
             mockAction m
         mockCounter m `shouldReturn` 23
@@ -396,8 +480,12 @@ spec = do
         r `shouldContain` "<span class=\"hspec-failure\">foo"
 
   describe "hspecResult" $ do
+    let
+      hspecResult args = withArgs ("--format=silent" : args) . H.hspecResult
+      hspecResult_ = hspecResult []
+
     it "returns a summary of the test run" $ do
-      silence . H.hspecResult $ do
+      hspecResult_ $ do
         H.it "foo" True
         H.it "foo" False
         H.it "foo" False
@@ -406,7 +494,7 @@ spec = do
       `shouldReturn` H.Summary 5 2
 
     it "treats uncaught exceptions as failure" $ do
-      silence . H.hspecResult  $ do
+      hspecResult_  $ do
         H.it "foobar" throwException
       `shouldReturn` H.Summary 1 1
 
@@ -423,15 +511,15 @@ spec = do
       r `shouldBe` ""
 
     it "does not let escape error thunks from failure messages" $ do
-      r <- silence . H.hspecResult $ do
-        H.it "some example" (H.Fail Nothing $ "foobar" ++ undefined)
+      r <- hspecResult_ $ do
+        H.it "some example" (H.Result "" $ H.Failure Nothing . H.Reason $ "foobar" ++ undefined)
       r `shouldBe` H.Summary 1 1
 
     it "runs specs in parallel" $ do
-      let n = 10
+      let n = 100
           t = 0.01
           dt = t * (fromIntegral n / 2)
-      r <- timeout dt . silence . withArgs ["-j", show n] . H.hspecResult . H.parallel $ do
+      r <- timeout dt . hspecResult ["-j", show n] . H.parallel $ do
         replicateM_ n (H.it "foo" $ sleep t)
       r `shouldBe` Just (H.Summary n 0)
 
@@ -446,8 +534,41 @@ spec = do
               current <- atomicModifyIORef currentRef $ \x -> let y = succ x in (y, y)
               atomicModifyIORef highRef $ \x -> (max x current, ())
             stop = atomicModifyIORef currentRef $ \x -> (pred x, ())
-        r <- withArgs ["-j", show j] . H.hspecResult . H.parallel $ do
+        r <- hspecResult ["-j", show j] . H.parallel $ do
           replicateM_ n $ H.it "foo" $ E.bracket_ start stop $ sleep t
         r `shouldBe` H.Summary n 0
         high <- readIORef highRef
         high `shouldBe` j
+
+  describe "rerunAll" $ do
+    let
+      report = FailureReport 0 0 0 0 [([], "foo")]
+      config = H.defaultConfig {H.configRerun = True, H.configRerunAllOnSuccess = True}
+      summary = H.Summary 1 0
+    context "with --rerun, --rerun-all-on-success, previous failures, on success" $ do
+      it "returns True" $ do
+        H.rerunAll config (Just report) summary `shouldBe` True
+
+    context "without --rerun" $ do
+      it "returns False" $ do
+        H.rerunAll config {H.configRerun = False} (Just report) summary `shouldBe` False
+
+    context "without --rerun-all-on-success" $ do
+      it "returns False" $ do
+        H.rerunAll config {H.configRerunAllOnSuccess = False} (Just report) summary `shouldBe` False
+
+    context "without previous failures" $ do
+      it "returns False" $ do
+        H.rerunAll config (Just report {failureReportPaths = []}) summary `shouldBe` False
+
+    context "without failure report" $ do
+      it "returns False" $ do
+        H.rerunAll config Nothing summary `shouldBe` False
+
+    context "on failure" $ do
+      it "returns False" $ do
+        H.rerunAll config (Just report) summary {H.summaryFailures = 1} `shouldBe` False
+  where
+    green  = setSGRCode [SetColor Foreground Dull Green]
+    red    = setSGRCode [SetColor Foreground Dull Red]
+    reset  = setSGRCode [Reset]
